#include <cstddef>
using std::size_t;

#include <cstdlib>
using std::atof;

#include <cassert>

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include <vector>
using std::vector;

#include <iostream>
using std::cerr;
using std::endl;

#include "mtrand.hpp"
mtrand R(1975);

#include "grid.hpp"

square_grid<cell> G(100);

class color
{
public:
	double red,green,blue;
	color() : red(0.2+0.8*R()), green(0.2+0.8*R()), blue(0.2+0.8*R()) {}
};

void display()
{
	static vector<color> color_list;
	const double d = 1.0/G.length();
	glClear(GL_COLOR_BUFFER_BIT);
	for (size_t i = 0; i < G.length(); ++i)
		for (size_t j = 0; j < G.length(); ++j)
			if (!G(i,j).is_removed)
			{
				const double x = 2.0*(i+0.5)/G.length() - 1.0;
				const double y = 2.0*(j+0.5)/G.length() - 1.0;
				const size_t k = G(i,j).cluster_membership-1;
				while (color_list.size() <= k)
					color_list.push_back(color());
				assert(k < color_list.size());
				glBegin(GL_POLYGON);
					glColor3f(color_list[k].red,color_list[k].blue,color_list[k].green); 
					glVertex2f(x-d,y-d);
					glVertex2f(x-d,y+d);
					glVertex2f(x+d,y+d);
					glVertex2f(x+d,y-d);
				glEnd();
				glFlush();
			}
	glutSwapBuffers();
}

void initialize_grid(double p)
{
	for (size_t i = 0; i < G.length(); ++i)
		for (size_t j = 0; j < G.length(); ++j)
		{
			G(i,j).cluster_membership = -1;
			G(i,j).is_removed = ( R() < p );
		}
}

void assign_membership(size_t i, size_t j, long int m)
{
	if (!G(i,j).is_removed and G(i,j).cluster_membership == -1)
	{
		G(i,j).cluster_membership = m;
		if (i+1 < G.length())
			assign_membership(i+1,j,m);
		if (i-1 < G.length())
			assign_membership(i-1,j,m);
		if (j+1 < G.length())
			assign_membership(i,j+1,m);
		if (j-1 < G.length())
			assign_membership(i,j-1,m);
	}
}

void build_clusters(void)
{
	long int m = 0;
	for (size_t i = 0; i < G.length(); ++i)
		for (size_t j = 0; j < G.length(); ++j)
			if (!G(i,j).is_removed and G(i,j).cluster_membership == -1)
				assign_membership(i,j,++m);
}

void bad()
{
	cerr << "Usage: perc2 (0 <= p < 1)" << endl;
	exit(1);
}

int main(int argc, char **argv)
{
	if (argc != 2) bad();
	const double p = atof(argv[1]);
	if (!(p >= 0.0 and p < 1.0)) bad();

	glutInit(&argc,argv);
	glutInitDisplayMode (GLUT_RGB | GLUT_DOUBLE);
	glutInitWindowSize (640, 640);
	glutInitWindowPosition (100, 100);
	glutCreateWindow("Percolation");
	glutDisplayFunc(display);
	
	initialize_grid(p);
	build_clusters();
	glutMainLoop();
}
