#include <cstddef>
using std::size_t;

#include <cstdlib>
using std::atoi;
using std::atof;

#include <cassert>

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include <vector>
using std::vector;

#include <iostream>
using std::cerr;
using std::cout;
using std::endl;

#include <iomanip>
using std::setw;

#include "mtrand.hpp"
mtrand R(1975);

#include "grid.hpp"

square_grid<cell> G;

void initialize_grid(double p)
{
	for (size_t i = 0; i < G.length(); ++i)
		for (size_t j = 0; j < G.length(); ++j)
		{
			G(i,j).cluster_membership = -1;
			G(i,j).is_removed = ( R() < p );
		}
}

long int assign_membership(size_t i, size_t j, long int m)
{
	if (!G(i,j).is_removed and G(i,j).cluster_membership == -1)
	{
		long int cluster_size = 1;
		G(i,j).cluster_membership = m;
		if (i+1 < G.length())
			cluster_size += assign_membership(i+1,j,m);
		if (i-1 < G.length())
			cluster_size += assign_membership(i-1,j,m);
		if (j+1 < G.length())
			cluster_size += assign_membership(i,j+1,m);
		if (j-1 < G.length())
			cluster_size += assign_membership(i,j-1,m);
		return cluster_size;
	}
	else
		return 0;
}

void build_clusters(unsigned long int &Nmax, double &N2, double &N4)
{
	long int m = 0;
	Nmax = 0;
	N2 = 0.0;
	N4 = 0.0;
	for (size_t i = 0; i < G.length(); ++i)
		for (size_t j = 0; j < G.length(); ++j)
			if (!G(i,j).is_removed and G(i,j).cluster_membership == -1)
			{
				const size_t Ncl = assign_membership(i,j,++m);
				if (Ncl > Nmax) Nmax = Ncl;
				const double x = Ncl;
				const double x2 = x*x;
				N2 += x2;
				N4 += x2*x2;
			}
}

void bad()
{
	cerr << "Usage: perc3 (0 < L < 500) (0 <= p < 1)" << endl;
	exit(1);
}

int main(int argc, char **argv)
{
	if (argc != 3) bad();
	const size_t L = atoi(argv[1]);
	if (!(L > 0 and L < 500)) bad();
	const double p = atof(argv[2]);
	if (!(p >= 0.0 and p < 1.0)) bad();
	G.set_length(L);

	const unsigned int Nsamples = 100;
	unsigned long int cluster_largest = 0;
	double cluster_2nd_moment = 0.0;
	double cluster_4th_moment = 0.0;
	for (unsigned int k = 0; k < Nsamples; ++k)
	{
		initialize_grid(p);
		unsigned long int Nmax;
		double N2, N4;
		build_clusters(Nmax,N2,N4);
		cluster_largest += Nmax;
		cluster_2nd_moment += N2;
		cluster_4th_moment += N4;
	}
	
	cout << setw(20) << double(cluster_largest)/Nsamples
	     << setw(20) << cluster_2nd_moment/Nsamples << "\t"
		 << setw(20) << cluster_4th_moment/Nsamples << endl;
}
